package fr.paristech.telecom.comasic.techweb.SearchEngine;
import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;
import java.util.SortedMap;
import java.util.SortedSet;
import java.util.TreeMap;
import java.util.TreeSet;
import java.util.Map.Entry;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class InvertedIndex {
	private SortedMap<String,SortedSet<ScoredDocument>> index=
		new TreeMap<String,SortedSet<ScoredDocument>>();
	
	static private Stemmer stemmer=new Stemmer();
	
	static private synchronized String stem(String word)
	{
		stemmer.add(word.toCharArray(),word.length());
		stemmer.stem();
		return stemmer.toString();	
	}
	
	public InvertedIndex(String filename, String stopWordsFilename)
	  throws IOException {
	
		Set<String> stopWords=new HashSet<String>();
		
		BufferedReader r=new BufferedReader(new FileReader(stopWordsFilename));
		String line;
		
		while((line=r.readLine())!=null) {
			stopWords.add(stem(line));
		}
		
		r=new BufferedReader(new FileReader (filename));
		
		Pattern alphaDigit = Pattern.compile("[\\p{L}\\p{N}]+");
		
		int nbDocuments=0;
		while((line=r.readLine())!=null) {
			String title=line;
			
			StringBuilder content=new StringBuilder();
			while((line=r.readLine())!=null && !line.equals(""))
				content.append(line+"\n");
			
			HashMap<String,Integer> occurrences=new HashMap<String,Integer>(); 
			
			int nbTokens=0;
			Matcher matcher=alphaDigit.matcher(content);
			while(matcher.find()) {
				String token=stem(matcher.group().toLowerCase());
				
				++nbTokens;
				
				if(stopWords.contains(token))
					continue;
				
				Integer nbOccurrences=occurrences.get(token);
				if(nbOccurrences!=null)
					occurrences.put(token, nbOccurrences+1);
				else
					occurrences.put(token,1);
			}
			
			for(Entry<String,Integer> e:occurrences.entrySet()) {
				String token=e.getKey();

				if(!index.containsKey(token))
					index.put(token,new TreeSet<ScoredDocument>());

				index.get(token).add(new ScoredDocument(title,((double)e.getValue())/nbTokens));
			
			}
			
			++nbDocuments;
			
			if(nbDocuments%1000==0) { 
				System.err.print(".");
			}
		}
		
		// Compute IDF
		for(SortedSet<ScoredDocument> s:index.values()) {
			int dc=s.size();
			
			for(ScoredDocument ti:s)
				ti.weight*=Math.log(nbDocuments/dc);
		}
	}
	
	SortedSet<ScoredDocument> executeQuery(String[] query, boolean conjunctive)
	{	
		Map<String,Double> scores=new HashMap<String,Double>();
		boolean first=true;
		
		for(String token:query) {
			SortedSet<ScoredDocument> partialResult=index.get(stem(token));
			
			if(partialResult==null) {
				if(conjunctive)
					return null;
				else 
					continue;
			} else {
				Map<String,Double> oldScores=new TreeMap<String,Double>(scores);
				
				if(conjunctive && !first)
					scores.clear();
				
				for(ScoredDocument sd:partialResult) {
					if(oldScores.containsKey(sd.document))
						scores.put(sd.document, oldScores.get(sd.document)+sd.weight);
					else if(!conjunctive || first)
						scores.put(sd.document,sd.weight);
				} 
			}
			
			first=false;
		}
		
		SortedSet<ScoredDocument> result=new TreeSet<ScoredDocument>();
		for(Entry<String,Double> e:scores.entrySet())
			result.add(new ScoredDocument(e.getKey(),e.getValue()));
		
		return result;
	}
	
	SortedSet<ScoredDocument> disjunctiveQuery(String[] query)
	{
		return executeQuery(query,false);
	}
	
	SortedSet<ScoredDocument> conjunctiveQuery(String[] query)
	{
		return executeQuery(query,true);
	}
}
