package fr.paristech.telecom.comasic.techweb.SearchEngine;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.ServletException;
 
import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.SortedSet;
 
import org.eclipse.jetty.server.Server;
import org.eclipse.jetty.server.Request;
import org.eclipse.jetty.server.handler.AbstractHandler;

public class WebInterface extends AbstractHandler
{
	private InvertedIndex index;
	
	WebInterface(InvertedIndex index) {
		super();
		this.index=index;
	}
	
    public void handle(String target,
                       Request baseRequest,
                       HttpServletRequest request,
                       HttpServletResponse response) 
        throws IOException, ServletException
    {
        response.setContentType("text/html;charset=utf-8");
        response.setStatus(HttpServletResponse.SC_OK);
        baseRequest.setHandled(true);
        PrintWriter writer=response.getWriter();
        
        String search=request.getParameter("search");
        if(search==null)
        	search="";
        
        String header=readFile("html/header.html");
        writer.print(header.replaceAll("DEFAULT",escape(search)));
        
        if(!search.isEmpty()) {
        	search=search.replaceAll("[^a-zA-Z0-9 ]","");
        	SortedSet<ScoredDocument> results=index.conjunctiveQuery(search.split(" "));
        	displayResults(writer, results, 20);
        }
        
        writer.print(readFile("html/footer.html"));
    }
 
    public static void main(String[] args) throws Exception
    {
    	InvertedIndex index=null;
		
		System.err.print("Loading index");
		try {
			index=new InvertedIndex("data/text","data/stop_words.txt");
		} catch(IOException e) {
			System.err.println("Cannot build the index.");
			System.exit(1);
		}
		System.err.println("");
		
        Server server = new Server(1234);
        server.setHandler(new WebInterface(index));
 
        server.start();
        server.join();
    }

    private static String readFile( String file ) throws IOException {
        BufferedReader reader = new BufferedReader( new FileReader (file));
        String         line = null;
        StringBuilder  stringBuilder = new StringBuilder();
        String         ls = System.getProperty("line.separator");
        while( ( line = reader.readLine() ) != null ) {
            stringBuilder.append( line );
            stringBuilder.append( ls );
        }
        return stringBuilder.toString();
    }
    

	static private void displayResults(PrintWriter writer,SortedSet<ScoredDocument> result, int limit) {
		writer.println("    <ul>");
		if (result != null) {
			int i=0;
			
			for (ScoredDocument sd : result) {
				if (++i == limit)
					break;
				String document = sd.document;

				writer.println("      <li><a href='http://simple.wikipedia.org/wiki/"
								+ document.replaceAll(" ", "_")
								+ "'>"
								+ sd.document
								+ "</a> ("
								+ sd.weight
								+ ")"
								+ "</li>");
			}
		}
		writer.println("    </ul>");
	}
	
	private static String escape(String s)
	{
		s = s.replaceAll("&", "&amp;");
		s = s.replaceAll("'", "&apos;");
		s = s.replaceAll("\"", "&quot;");
		s = s.replaceAll("<", "&lt;");
		return s;
	}
}